#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2024-10-14 00:28:03 (ywatanabe)"
# File: 2024-10-07 22:17:30

try:
    from ._typing import List, Tuple, Dict, Any, Union, Sequence, Literal, Optional, Iterable, Generator, ArrayLike
except ImportError as e:
    warnings.warn(f"Warning: Failed to import from ._typing.")

# from ._typing import List, Tuple, Dict, Any, Union, Sequence, Literal, Optional, Iterable, Generator, ArrayLike
