#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Time-stamp: "2024-10-24 18:40:05 (ywatanabe)"

from ._mv import mv
# try:
#     from ._mv import mv
# except ImportError as e:
#     warnings.warn(f"Warning: Failed to import mv from ._mv.")
